"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFunctionMap = exports.AWSLambdaAPI = void 0;
const radash_1 = __importDefault(require("radash"));
const pulumi = __importStar(require("@pulumi/pulumi"));
const aws = __importStar(require("@pulumi/aws"));
const awsx = __importStar(require("@pulumi/awsx"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const cmdish_1 = __importDefault(require("cmdish"));
/**
 * Static website using Amazon S3, CloudFront, and Route53.
 */
class AWSLambdaAPI extends pulumi.ComponentResource {
    constructor(name, args, opts) {
        const inputs = {
            options: opts
        };
        super('exo:components:aws-lambda-api', name, inputs, opts);
        const apiName = radash_1.default.dashCase(`${name}-api`);
        const iamRoleName = radash_1.default.dashCase(`${name}-iam-lambda`);
        /**
         * aws component will add a dash and 7 rand chars to our name and
         * the aws limit is 64. We need to trim it if its over
         * limit = 64 - (dash = 1) - (rand = 7)
         */
        const lambdaName = (module, func) => {
            const n = radash_1.default.dashCase(`${name}-${module}-${func}`);
            return n.length > 56
                ? n.slice(0, 56 - n.length)
                : n;
        };
        //
        //  DETERMINE API SHAPE FROM SOURCE
        //
        const functions = getFunctionMap({
            path: args.sourceDir,
            ext: args.sourceExt
        });
        //
        //  CREATE NEEDED ROLE/POLICY
        //
        const iamForLambda = new aws.iam.Role(iamRoleName, {
            assumeRolePolicy: `{
      "Version": "2012-10-17",
      "Statement": [
        {
          "Action": "sts:AssumeRole",
          "Principal": {
            "Service": "lambda.amazonaws.com"
          },
          "Effect": "Allow",
          "Sid": ""
        }
      ]
    }`
        }, opts);
        const lambdaLoggingPolicy = new aws.iam.Policy("lambdaLogging", {
            path: "/",
            description: "IAM policy for logging from a lambda",
            policy: `{
      "Version": "2012-10-17",
      "Statement": [
        {
          "Action": [
            "logs:CreateLogGroup",
            "logs:CreateLogStream",
            "logs:PutLogEvents"
          ],
          "Resource": "arn:aws:logs:*:*:*",
          "Effect": "Allow"
        }
      ]
    }
    `
        }, opts);
        const loggingPolicyAttachment = new aws.iam.RolePolicyAttachment("lambdaLogs", {
            role: iamForLambda.name,
            policyArn: lambdaLoggingPolicy.arn,
        }, opts);
        //
        //  CREATE LAMBDA FOR EACH FUNCTION
        //
        const zipSource = new pulumi.asset.FileArchive(buildLambdaZip(args));
        const envvars = args.environmentVariables.reduce((acc, envvar) => (Object.assign(Object.assign({}, acc), { [envvar.name]: envvar.value })), {});
        const lambdas = functions.map((func) => {
            const lambda = new aws.lambda.Function(lambdaName(func.module, func.function), {
                code: zipSource,
                role: iamForLambda.arn,
                handler: `modules/${func.module}/${func.function}.default`,
                runtime: args.runtime,
                timeout: args.timeout,
                memorySize: args.memory,
                environment: {
                    variables: Object.assign(Object.assign({}, envvars), { EXOBASE_MODULE: func.module, EXOBASE_FUNCTION: func.function })
                }
            }, Object.assign(Object.assign({}, opts), { dependsOn: [
                    loggingPolicyAttachment
                ] }));
            return Object.assign(Object.assign({}, func), { lambda });
        });
        this.lambdas = lambdas.map(l => l.lambda);
        //
        //  CREATE API GATEWAY ENDPOINTS FOR EACH FUNCTION
        //
        const api = new awsx.apigateway.API(apiName, {
            routes: lambdas.map(lambda => ({
                path: `/${lambda.module}/${lambda.function}`,
                method: 'ANY',
                eventHandler: lambda.lambda,
            })),
            stageName: 'api'
        }, opts);
        this.api = api;
        //
        //  SETUP CUSTOM DOMAIN (if specified)
        //
        if (args.domain) {
            const cert = pulumi.output(aws.acm.getCertificate({
                domain: args.domain.subdomain
                    ? `*.${args.domain.domain}`
                    : args.domain.domain,
                mostRecent: true,
                types: ["AMAZON_ISSUED"]
            }, opts));
            const domainName = new aws.apigateway.DomainName(radash_1.default.dashCase(args.domain.fqd), {
                certificateArn: cert.arn,
                domainName: args.domain.fqd
            }, opts);
            new aws.apigateway.BasePathMapping(radash_1.default.dashCase(`${args.domain.fqd}-mapping`), {
                restApi: api.restAPI,
                stageName: api.stage.stageName,
                domainName: domainName.domainName
            }, opts);
            const zone = pulumi.output(aws.route53.getZone({
                name: args.domain.domain,
                privateZone: false
            }, opts));
            /**
             * Really odd use of cloud front args here. See this terraform
             * doc for clarification. TLDR; api gateway is using cloud front
             * under the hood.
             * https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/api_gateway_domain_name
             */
            this.aRecord = new aws.route53.Record(radash_1.default.dashCase(`${args.domain.fqd}-record`), {
                zoneId: zone.id,
                name: domainName.domainName,
                type: "A",
                aliases: [{
                        name: domainName.cloudfrontDomainName,
                        zoneId: domainName.cloudfrontZoneId,
                        evaluateTargetHealth: false
                    }]
            }, Object.assign(Object.assign({}, opts), { dependsOn: [
                    domainName
                ] }));
        }
    }
}
exports.AWSLambdaAPI = AWSLambdaAPI;
/**
 * Looks in ./src/modules for your modules and
 * functions. Returns their names and locations
 * as an array.
 */
function getFunctionMap({ path: rootPath, ext }) {
    const relPath = (rel) => path_1.default.join(rootPath, rel);
    const modules = fs_extra_1.default.readdirSync(relPath('/src/modules'), { withFileTypes: true })
        .filter(item => item.isDirectory())
        .map(m => {
        return fs_extra_1.default.readdirSync(relPath(`/src/modules/${m.name}`), { withFileTypes: true })
            .filter(item => !item.isDirectory())
            .filter(item => item.name.endsWith(`.${ext}`))
            .map(tsFile => {
            const funcName = tsFile.name.replace(`.${ext}`, '');
            return {
                function: funcName,
                module: m.name,
                paths: {
                    file: relPath(`/src/modules/${m.name}/${tsFile.name}`),
                    import: relPath(`/src/modules/${m.name}/${funcName}`)
                }
            };
        });
    });
    return radash_1.default.flat(modules);
}
exports.getFunctionMap = getFunctionMap;
const buildLambdaZip = (args) => __awaiter(void 0, void 0, void 0, function* () {
    const zip = `${args.sourceDir}/aws-lambda-api.zip`;
    //
    // Build the source
    //
    const [err] = yield (0, cmdish_1.default)(args.buildCommand, {
        cwd: args.sourceDir
    });
    if (err)
        throw err;
    //
    // Generate new zip
    //
    yield (0, cmdish_1.default)(`zip -q -r ${zip} *`, {
        cwd: `${args.sourceDir}${args.distDirName}`
    });
    return zip;
});
//# sourceMappingURL=index.js.map